@@ -191,6 +191,7 @@ class ConfigMain::Impl {
     OptionBool debug_solver{false};
     OptionStringList installonlypkgs{INSTALLONLYPKGS};
     OptionStringList group_package_types{GROUP_PACKAGE_TYPES};
+    OptionStringList optional_metadata_types{std::vector<std::string>{}};
 
     OptionNumber<std::uint32_t> installonly_limit{3, 0,
         [](const std::string & value)->std::uint32_t{
@@ -283,6 +284,7 @@ class ConfigMain::Impl {
     OptionString comment{nullptr};
     OptionBool downloadonly{false}; // runtime only option
     OptionBool ignorearch{false};
+    OptionEnum<std::string> persistence{"auto", {"auto", "persist", "transient"}};
     OptionString module_platform_id{nullptr};
     OptionBool module_stream_switch{false};
     OptionBool module_obsoletes{false};
@@ -313,8 +315,9 @@ class ConfigMain::Impl {
         }
     };
 
-    OptionStringList protected_packages{resolveGlobs("dnf glob:/etc/yum/protected.d/*.conf " \
-                                          "glob:/etc/dnf/protected.d/*.conf")};
+    OptionStringList protected_packages{resolveGlobs("dnf " \
+                                                     "glob:/etc/yum/protected.d/*.conf " \
+                                                     "glob:/etc/dnf/protected.d/*.conf")};
     OptionString username{""};
     OptionString password{""};
     OptionBool gpgcheck{false};
@@ -359,7 +362,7 @@ class ConfigMain::Impl {
     OptionBool proxy_sslverify{true};
     OptionString proxy_sslclientcert{""};
     OptionString proxy_sslclientkey{""};
-    OptionBool deltarpm{true};
+    OptionBool deltarpm{false};
     OptionNumber<std::uint32_t> deltarpm_percentage{75};
     OptionBool skip_if_unavailable{false};
     OptionBool sslverifystatus{false};
@@ -398,6 +401,7 @@ ConfigMain::Impl::Impl(Config & owner)
     );
 
     owner.optBinds().add("group_package_types", group_package_types);
+    owner.optBinds().add("optional_metadata_types", optional_metadata_types);
     owner.optBinds().add("installonly_limit", installonly_limit);
 
     owner.optBinds().add("tsflags", tsflags,
@@ -455,6 +459,7 @@ ConfigMain::Impl::Impl(Config & owner)
     owner.optBinds().add("user_agent", user_agent);
     owner.optBinds().add("countme", countme);
     owner.optBinds().add("protect_running_kernel", protect_running_kernel);
+    owner.optBinds().add("persistence", persistence);
 
     // Repo main config
 
@@ -485,7 +490,19 @@ ConfigMain::Impl::Impl(Config & owner)
     );
 
     owner.optBinds().add("exclude_from_weak_autodetect", exclude_from_weak_autodetect);
-    owner.optBinds().add("proxy", proxy);
+
+    owner.optBinds().add("proxy", proxy,
+        [&](Option::Priority priority, const std::string & value){
+            auto tmpValue(value);
+            for (auto & ch : tmpValue)
+                ch = std::tolower(ch);
+            if (tmpValue == "_none_")
+                proxy.set(priority, "");
+            else
+                proxy.set(priority, value);
+        }, nullptr, false
+    );
+
     owner.optBinds().add("proxy_username", proxy_username);
     owner.optBinds().add("proxy_password", proxy_password);
     owner.optBinds().add("proxy_auth_method", proxy_auth_method);
@@ -551,6 +568,7 @@ OptionStringList & ConfigMain::reposdir() { return pImpl->reposdir; }
 OptionBool & ConfigMain::debug_solver() { return pImpl->debug_solver; }
 OptionStringList & ConfigMain::installonlypkgs() { return pImpl->installonlypkgs; }
 OptionStringList & ConfigMain::group_package_types() { return pImpl->group_package_types; }
+OptionStringList & ConfigMain::optional_metadata_types() { return pImpl->optional_metadata_types; }
 OptionNumber<std::uint32_t> & ConfigMain::installonly_limit() { return pImpl->installonly_limit; }
 OptionStringList & ConfigMain::tsflags() { return pImpl->tsflags; }
 OptionBool & ConfigMain::assumeyes() { return pImpl->assumeyes; }
@@ -597,6 +615,7 @@ OptionPath & ConfigMain::destdir() { return pImpl->destdir; }
 OptionString & ConfigMain::comment() { return pImpl->comment; }
 OptionBool & ConfigMain::downloadonly() { return pImpl->downloadonly; }
 OptionBool & ConfigMain::ignorearch() { return pImpl->ignorearch; }
+OptionEnum<std::string> & ConfigMain::persistence() { return pImpl->persistence; }
 
 OptionString & ConfigMain::module_platform_id() { return pImpl->module_platform_id; }
 OptionBool & ConfigMain::module_stream_switch() { return pImpl->module_stream_switch; }
