@@ -34,6 +34,7 @@
  */
 
 #include "conf/OptionBool.hpp"
+#include "conf/ConfigParser.hpp"
 
 #include "dnf-context.hpp"
 #include "hy-repo-private.hpp"
@@ -45,6 +46,7 @@
 #include <glib/gstdio.h>
 #include "hy-util.h"
 #include <librepo/librepo.h>
+#include <librepo/url_substitution.h>
 #include <rpm/rpmts.h>
 #include <librepo/yum.h>
 
@@ -242,14 +244,17 @@ static gchar *
 dnf_repo_substitute(DnfRepo *repo, const gchar *url)
 {
     DnfRepoPrivate *priv = GET_PRIVATE(repo);
-    char *tmp;
-    gchar *substituted;
 
-    /* do a little dance so we can use g_free() rather than lr_free() */
-    tmp = lr_url_substitute(url, priv->urlvars);
-    substituted = g_strdup(tmp);
-    lr_free(tmp);
+    std::map<std::string, std::string> substitutions;
+    for (LrUrlVars *elem = priv->urlvars; elem; elem = g_slist_next(elem)) {
+        const auto * pair = static_cast<LrVar*>(elem->data);
+        substitutions.insert({std::string{pair->var}, std::string{pair->val}});
+    }
 
+    std::string tmp{url};
+    libdnf::ConfigParser::substitute(tmp, substitutions);
+
+    auto * substituted = g_strdup(tmp.c_str());
     return substituted;
 }
 
@@ -1187,6 +1192,8 @@ dnf_repo_setup(DnfRepo *repo, GError **error) try
     DnfRepoEnabled enabled = DNF_REPO_ENABLED_NONE;
     g_autofree gchar *basearch = NULL;
     g_autofree gchar *release = NULL;
+    g_autofree gchar *release_major = NULL;
+    g_autofree gchar *release_minor = NULL;
 
     basearch = g_key_file_get_string(priv->keyfile, "general", "arch", NULL);
     if (basearch == NULL)
@@ -1199,8 +1206,14 @@ dnf_repo_setup(DnfRepo *repo, GError **error) try
         return FALSE;
     }
     release = g_key_file_get_string(priv->keyfile, "general", "version", NULL);
-    if (release == NULL)
+    if (release == NULL) {
         release = g_strdup(dnf_context_get_release_ver(priv->context));
+        release_major = g_strdup(dnf_context_get_release_ver_major(priv->context));
+        release_minor = g_strdup(dnf_context_get_release_ver_minor(priv->context));
+    } else {
+        dnf_split_releasever(release, &release_major, &release_minor);
+    }
+
     if (release == NULL) {
         g_set_error_literal(error,
                             DNF_ERROR,
@@ -1215,8 +1228,9 @@ dnf_repo_setup(DnfRepo *repo, GError **error) try
     if (!lr_handle_setopt(priv->repo_handle, error, LRO_INTERRUPTIBLE, 0L))
         return FALSE;
     priv->urlvars = lr_urlvars_set(priv->urlvars, "releasever", release);
+    priv->urlvars = lr_urlvars_set(priv->urlvars, "releasever_major", release_major);
+    priv->urlvars = lr_urlvars_set(priv->urlvars, "releasever_minor", release_minor);
     priv->urlvars = lr_urlvars_set(priv->urlvars, "basearch", basearch);
-
     /* Call libdnf::dnf_context_load_vars(priv->context); only when values not in cache.
      * But what about if variables on disk change during long running programs (PackageKit daemon)?
      * if (!libdnf::dnf_context_get_vars_cached(priv->context))
