@@ -602,7 +602,7 @@ dnf_transaction_ts_progress_cb(const void *arg,
 
             /* map to correct action code */
             action = dnf_package_get_action(pkg);
-            if (action == DNF_STATE_ACTION_UNKNOWN)
+            if (action == DNF_STATE_ACTION_UNKNOWN || action == DNF_STATE_ACTION_REINSTALL)
                 action = DNF_STATE_ACTION_INSTALL;
 
             /* set the pkgid if not already set */
@@ -641,7 +641,7 @@ dnf_transaction_ts_progress_cb(const void *arg,
 
             /* map to correct action code */
             action = dnf_package_get_action(pkg);
-            if (action == DNF_STATE_ACTION_UNKNOWN)
+            if (action == DNF_STATE_ACTION_UNKNOWN || action == DNF_STATE_ACTION_REINSTALL)
                 action = DNF_STATE_ACTION_REMOVE;
 
             /* remove start */
@@ -716,7 +716,7 @@ dnf_transaction_ts_progress_cb(const void *arg,
 
             /* map to correct action code */
             action = dnf_package_get_action(pkg);
-            if (action == DNF_STATE_ACTION_UNKNOWN)
+            if (action == DNF_STATE_ACTION_UNKNOWN || action == DNF_STATE_ACTION_REINSTALL)
                 action = DNF_STATE_ACTION_REMOVE;
 
             dnf_state_set_package_progress(
@@ -868,7 +868,7 @@ _get_current_time()
  * avoid the lookup in the rpmdb.
  **/
 static void
-_history_write_item(DnfPackage *pkg, libdnf::Swdb *swdb, libdnf::TransactionItemAction action)
+_history_write_item(DnfPackage *pkg, libdnf::Swdb *swdb, libdnf::TransactionItemAction action, libdnf::TransactionItemReason reason)
 {
     auto rpm = swdb->createRPMItem();
     rpm->setName(dnf_package_get_name(pkg));
@@ -878,8 +878,9 @@ _history_write_item(DnfPackage *pkg, libdnf::Swdb *swdb, libdnf::TransactionItem
     rpm->setArch(dnf_package_get_arch(pkg));
     rpm->save();
 
-    libdnf::TransactionItemReason reason =
-        swdb->resolveRPMTransactionItemReason(rpm->getName(), rpm->getArch(), -2);
+    if (reason == libdnf::TransactionItemReason::UNKNOWN) {
+        reason = swdb->resolveRPMTransactionItemReason(rpm->getName(), rpm->getArch(), -2);
+    }
 
     auto transItem = swdb->addItem(
         std::dynamic_pointer_cast< libdnf::Item >(rpm), dnf_package_get_reponame(pkg), action, reason);
@@ -1221,8 +1222,12 @@ dnf_transaction_commit(DnfTransaction *transaction, HyGoal goal, DnfState *state
         filename = dnf_package_get_filename(pkg);
         allow_untrusted = (priv->flags & DNF_TRANSACTION_FLAG_ONLY_TRUSTED) == 0;
         is_update = action == DNF_STATE_ACTION_UPDATE || action == DNF_STATE_ACTION_DOWNGRADE;
-        ret = dnf_rpmts_add_install_filename2(
-            priv->ts, filename, allow_untrusted, is_update, pkg, error);
+        if (action == DNF_STATE_ACTION_REINSTALL) {
+            ret = dnf_rpmts_add_reinstall_filename(priv->ts, filename, allow_untrusted, error);
+        } else {
+            ret = dnf_rpmts_add_install_filename2(
+                priv->ts, filename, allow_untrusted, is_update, pkg, error);
+        }
         if (!ret)
             goto out;
 
@@ -1236,8 +1241,23 @@ dnf_transaction_commit(DnfTransaction *transaction, HyGoal goal, DnfState *state
             swdbAction = libdnf::TransactionItemAction::REINSTALL;
         }
 
+        libdnf::TransactionItemReason swdbReason;
+        if (swdbAction != libdnf::TransactionItemAction::INSTALL) {
+            /* Placeholder to be fetched later from history */
+            swdbReason = libdnf::TransactionItemReason::UNKNOWN;
+        } else switch (hy_goal_get_reason(goal, pkg)) {
+            case HY_REASON_USER:
+                swdbReason = libdnf::TransactionItemReason::USER;
+                break;
+            case HY_REASON_DEP:
+                swdbReason = libdnf::TransactionItemReason::DEPENDENCY;
+                break;
+            default:
+                swdbReason = libdnf::TransactionItemReason::UNKNOWN;
+        }
+
         // add item to swdb transaction
-        _history_write_item(pkg, swdb, swdbAction);
+        _history_write_item(pkg, swdb, swdbAction, swdbReason);
 
         /* this section done */
         ret = dnf_state_done(state_local, error);
@@ -1277,7 +1297,7 @@ dnf_transaction_commit(DnfTransaction *transaction, HyGoal goal, DnfState *state
                 swdbAction = libdnf::TransactionItemAction::DOWNGRADED;
             }
         }
-        _history_write_item(pkg, swdb, swdbAction);
+        _history_write_item(pkg, swdb, swdbAction, libdnf::TransactionItemReason::UNKNOWN);
     }
 
     /* add anything that gets obsoleted to a helper array which is used to
@@ -1329,11 +1349,20 @@ dnf_transaction_commit(DnfTransaction *transaction, HyGoal goal, DnfState *state
             }
 
             // TODO SWDB add pkg_tmp replaced_by pkg
-            _history_write_item(pkg_tmp, swdb, swdbAction);
+            _history_write_item(pkg_tmp, swdb, swdbAction, libdnf::TransactionItemReason::UNKNOWN);
         }
         g_ptr_array_unref(pkglist);
     }
 
+    /* add reinstalled packages to a helper array which is used to
+     * map removed packages auto-added by rpm to actual DnfPackage's */
+    pkglist = dnf_goal_get_packages(goal, DNF_PACKAGE_INFO_REINSTALL, -1);
+    for (i = 0; i < pkglist->len; i++) {
+        pkg_tmp = static_cast< DnfPackage * >(g_ptr_array_index(pkglist, i));
+        g_ptr_array_add(priv->remove_helper, g_object_ref(pkg_tmp));
+    }
+    g_ptr_array_unref(pkglist);
+
     /* this section done */
     ret = dnf_state_done(state, error);
     if (!ret)
