@@ -47,6 +47,29 @@ extern "C" {
 #include "IdQueue.hpp"
 #include "../utils/filesystem.hpp"
 
+namespace {
+
+std::string pkgSolvid2str(Pool * pool, Id source)
+{
+    return pool_solvid2str(pool, source);
+}
+
+std::string moduleSolvid2str(Pool * pool, Id source)
+{
+    std::ostringstream ss;
+    auto * solvable = pool_id2solvable(pool, source);
+    // Add name:stream
+    ss << solvable_lookup_str(solvable, SOLVABLE_DESCRIPTION);
+    //Add version
+    ss << ":" << pool_id2str(pool, solvable->evr);
+    // Add original context
+    ss << ":" << solvable_lookup_str(solvable, SOLVABLE_SUMMARY);
+    ss << "." << pool_id2str(pool, solvable->arch);
+    return ss.str();
+}
+
+}
+
 namespace libdnf {
 
 enum {NO_MATCH=1, MULTIPLE_MATCH_OBJECTS, INCORECT_COMPARISON_TYPE};
@@ -64,133 +87,144 @@ enum {RULE_DISTUPGRADE=1, RULE_INFARCH, RULE_UPDATE, RULE_JOB, RULE_JOB_UNSUPPOR
 };
 
 static const std::map<int, const char *> PKG_PROBLEMS_DICT = {
-    {RULE_DISTUPGRADE, _(" does not belong to a distupgrade repository")},
-    {RULE_INFARCH, _(" has inferior architecture")},
-    {RULE_UPDATE, _("problem with installed package ")},
-    {RULE_JOB, _("conflicting requests")},
-    {RULE_JOB_UNSUPPORTED, _("unsupported request")},
-    {RULE_JOB_NOTHING_PROVIDES_DEP, _("nothing provides requested ")},
-    {RULE_JOB_UNKNOWN_PACKAGE, _("package %s does not exist")},
-    {RULE_JOB_PROVIDED_BY_SYSTEM, _(" is provided by the system")},
-    {RULE_PKG, _("some dependency problem")},
-    {RULE_BEST_1, _("cannot install the best update candidate for package ")},
-    {RULE_BEST_2, _("cannot install the best candidate for the job")},
-    {RULE_PKG_NOT_INSTALLABLE_1, _("package %s is filtered out by modular filtering")},
-    {RULE_PKG_NOT_INSTALLABLE_2, _("package %s does not have a compatible architecture")},
-    {RULE_PKG_NOT_INSTALLABLE_3, _("package %s is not installable")},
-    {RULE_PKG_NOT_INSTALLABLE_4, _("package %s is filtered out by exclude filtering")},
-    {RULE_PKG_NOTHING_PROVIDES_DEP, _("nothing provides %s needed by %s")},
-    {RULE_PKG_SAME_NAME, _("cannot install both %s and %s")},
-    {RULE_PKG_CONFLICTS, _("package %s conflicts with %s provided by %s")},
-    {RULE_PKG_OBSOLETES, _("package %s obsoletes %s provided by %s")},
-    {RULE_PKG_INSTALLED_OBSOLETES, _("installed package %s obsoletes %s provided by %s")},
-    {RULE_PKG_IMPLICIT_OBSOLETES, _("package %s implicitly obsoletes %s provided by %s")},
-    {RULE_PKG_REQUIRES, _("package %s requires %s, but none of the providers can be installed")},
-    {RULE_PKG_SELF_CONFLICT, _("package %s conflicts with %s provided by itself")},
-    {RULE_YUMOBS, _("both package %s and %s obsolete %s")}
+    {RULE_DISTUPGRADE, M_("%s from %s  does not belong to a distupgrade repository")},
+    {RULE_INFARCH, M_("%s from %s  has inferior architecture")},
+    {RULE_UPDATE, M_("problem with installed package ")},
+    {RULE_JOB, M_("conflicting requests")},
+    {RULE_JOB_UNSUPPORTED, M_("unsupported request")},
+    {RULE_JOB_NOTHING_PROVIDES_DEP, M_("nothing provides requested ")},
+    {RULE_JOB_UNKNOWN_PACKAGE, M_("package %s does not exist")},
+    {RULE_JOB_PROVIDED_BY_SYSTEM, M_(" is provided by the system")},
+    {RULE_PKG, M_("some dependency problem")},
+    {RULE_BEST_1, M_("cannot install the best update candidate for package ")},
+    {RULE_BEST_2, M_("cannot install the best candidate for the job")},
+    {RULE_PKG_NOT_INSTALLABLE_1, M_("package %s from %s is filtered out by modular filtering")},
+    {RULE_PKG_NOT_INSTALLABLE_2, M_("package %s from %s does not have a compatible architecture")},
+    {RULE_PKG_NOT_INSTALLABLE_3, M_("package %s from %s is not installable")},
+    {RULE_PKG_NOT_INSTALLABLE_4, M_("package %s from %s is filtered out by exclude filtering")},
+    {RULE_PKG_NOTHING_PROVIDES_DEP, M_("nothing provides %s needed by %s from %s")},
+    {RULE_PKG_SAME_NAME, M_("cannot install both %s from %s and %s from %s")},
+    {RULE_PKG_CONFLICTS, M_("package %s from %s conflicts with %s provided by %s from %s")},
+    {RULE_PKG_OBSOLETES, M_("package %s from %s obsoletes %s provided by %s from %s")},
+    {RULE_PKG_INSTALLED_OBSOLETES, M_("installed package %s obsoletes %s provided by %s from %s")},
+    {RULE_PKG_IMPLICIT_OBSOLETES, M_("package %s from %s implicitly obsoletes %s provided by %s from %s")},
+    {RULE_PKG_REQUIRES, M_("package %s from %s requires %s, but none of the providers can be installed")},
+    {RULE_PKG_SELF_CONFLICT, M_("package %s from %s conflicts with %s provided by itself")},
+    {RULE_YUMOBS, M_("both package %s from %s and %s from %s obsolete %s")}
 };
 
 static const std::map<int, const char *> MODULE_PROBLEMS_DICT = {
-    {RULE_DISTUPGRADE, _(" does not belong to a distupgrade repository")},
-    {RULE_INFARCH, _(" has inferior architecture")},
-    {RULE_UPDATE, _("problem with installed module ")},
-    {RULE_JOB, _("conflicting requests")},
-    {RULE_JOB_UNSUPPORTED, _("unsupported request")},
-    {RULE_JOB_NOTHING_PROVIDES_DEP, _("nothing provides requested ")},
-    {RULE_JOB_UNKNOWN_PACKAGE, _("module %s does not exist")},
-    {RULE_JOB_PROVIDED_BY_SYSTEM, _(" is provided by the system")},
-    {RULE_PKG, _("some dependency problem")},
-    {RULE_BEST_1, _("cannot install the best update candidate for module ")},
-    {RULE_BEST_2, _("cannot install the best candidate for the job")},
-    {RULE_PKG_NOT_INSTALLABLE_1, _("module %s is disabled")},
-    {RULE_PKG_NOT_INSTALLABLE_2, _("module %s does not have a compatible architecture")},
-    {RULE_PKG_NOT_INSTALLABLE_3, _("module %s is not installable")},
-    {RULE_PKG_NOT_INSTALLABLE_4, _("module %s is disabled")},
-    {RULE_PKG_NOTHING_PROVIDES_DEP, _("nothing provides %s needed by module %s")},
-    {RULE_PKG_SAME_NAME, _("cannot install both modules %s and %s")},
-    {RULE_PKG_CONFLICTS, _("module %s conflicts with %s provided by %s")},
-    {RULE_PKG_OBSOLETES, _("module %s obsoletes %s provided by %s")},
-    {RULE_PKG_INSTALLED_OBSOLETES, _("installed module %s obsoletes %s provided by %s")},
-    {RULE_PKG_IMPLICIT_OBSOLETES, _("module %s implicitly obsoletes %s provided by %s")},
-    {RULE_PKG_REQUIRES, _("module %s requires %s, but none of the providers can be installed")},
-    {RULE_PKG_SELF_CONFLICT, _("module %s conflicts with %s provided by itself")},
-    {RULE_YUMOBS, _("both module %s and %s obsolete %s")}
+    {RULE_DISTUPGRADE, M_("%s from %s does not belong to a distupgrade repository")},
+    {RULE_INFARCH, M_("%s from %s has inferior architecture")},
+    {RULE_UPDATE, M_("problem with installed module ")},
+    {RULE_JOB, M_("conflicting requests")},
+    {RULE_JOB_UNSUPPORTED, M_("unsupported request")},
+    {RULE_JOB_NOTHING_PROVIDES_DEP, M_("nothing provides requested ")},
+    {RULE_JOB_UNKNOWN_PACKAGE, M_("module %s does not exist")},
+    {RULE_JOB_PROVIDED_BY_SYSTEM, M_(" is provided by the system")},
+    {RULE_PKG, M_("some dependency problem")},
+    {RULE_BEST_1, M_("cannot install the best update candidate for module ")},
+    {RULE_BEST_2, M_("cannot install the best candidate for the job")},
+    {RULE_PKG_NOT_INSTALLABLE_1, M_("module %s from %s is disabled")},
+    {RULE_PKG_NOT_INSTALLABLE_2, M_("module %s from %s does not have a compatible architecture")},
+    {RULE_PKG_NOT_INSTALLABLE_3, M_("module %s from %s is not installable")},
+    {RULE_PKG_NOT_INSTALLABLE_4, M_("module %s from %s is disabled")},
+    {RULE_PKG_NOTHING_PROVIDES_DEP, M_("nothing provides %s needed by module %s from %s")},
+    {RULE_PKG_SAME_NAME, M_("cannot install both modules %s from %s and %s from %s")},
+    {RULE_PKG_CONFLICTS, M_("module %s from %s conflicts with %s provided by %s from %s")},
+    {RULE_PKG_OBSOLETES, M_("module %s from %s obsoletes %s provided by %s from %s")},
+    {RULE_PKG_INSTALLED_OBSOLETES, M_("installed module %s obsoletes %s provided by %s from %s")},
+    {RULE_PKG_IMPLICIT_OBSOLETES, M_("module %s from %s implicitly obsoletes %s provided by %s from %s")},
+    {RULE_PKG_REQUIRES, M_("module %s from %s requires %s, but none of the providers can be installed")},
+    {RULE_PKG_SELF_CONFLICT, M_("module %s from %s conflicts with %s provided by itself")},
+    {RULE_YUMOBS, M_("both module %s from %s and %s from %s obsolete %s")}
 };
 
 static std::string
 libdnf_problemruleinfo2str(libdnf::PackageSet * modularExclude, Solver *solv, SolverRuleinfo type, Id source, Id target,
-    Id dep, std::map<int, const char *> problemDict)
+    Id dep, bool pkgs)
 {
-    Pool *pool = solv->pool;
+    const std::map<int, const char *> & problemDict = pkgs ? PKG_PROBLEMS_DICT : MODULE_PROBLEMS_DICT;
+    const auto solvid2str = pkgs ? pkgSolvid2str : moduleSolvid2str;
+
+    Pool * const pool = solv->pool;
     Solvable *ss;
     switch (type) {
         case SOLVER_RULE_DISTUPGRADE:
-            return std::string(pool_solvid2str(pool, source)) + problemDict[RULE_DISTUPGRADE];
+            return tfm::format(TM_(problemDict.at(RULE_DISTUPGRADE), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name);
         case SOLVER_RULE_INFARCH:
-            return std::string(pool_solvid2str(pool, source)) + problemDict[RULE_INFARCH];
+            return tfm::format(TM_(problemDict.at(RULE_DISTUPGRADE), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name);
         case SOLVER_RULE_UPDATE:
-            return std::string(problemDict[RULE_UPDATE]) + pool_solvid2str(pool, source);
+            return std::string(TM_(problemDict.at(RULE_UPDATE), 1)) + solvid2str(pool, source);
         case SOLVER_RULE_JOB:
-            return std::string(problemDict[RULE_JOB]);
+            return std::string(TM_(problemDict.at(RULE_JOB), 1));
         case SOLVER_RULE_JOB_UNSUPPORTED:
-            return std::string(problemDict[RULE_JOB_UNSUPPORTED]);
+            return std::string(TM_(problemDict.at(RULE_JOB_UNSUPPORTED), 1));
         case SOLVER_RULE_JOB_NOTHING_PROVIDES_DEP:
-            return std::string(problemDict[RULE_JOB_NOTHING_PROVIDES_DEP]) + pool_dep2str(pool,
-                                                                                          dep);
+            return std::string(TM_(problemDict.at(RULE_JOB_NOTHING_PROVIDES_DEP), 1)) + pool_dep2str(pool, dep);
         case SOLVER_RULE_JOB_UNKNOWN_PACKAGE:
-            return tfm::format(problemDict[RULE_JOB_UNKNOWN_PACKAGE], pool_dep2str(pool, dep));
+            return tfm::format(TM_(problemDict.at(RULE_JOB_UNKNOWN_PACKAGE), 1), pool_dep2str(pool, dep));
         case SOLVER_RULE_JOB_PROVIDED_BY_SYSTEM:
-            return std::string(pool_dep2str(pool, dep)) + problemDict[RULE_JOB_PROVIDED_BY_SYSTEM]; 
+            return std::string(pool_dep2str(pool, dep)) + TM_(problemDict.at(RULE_JOB_PROVIDED_BY_SYSTEM), 1);
         case SOLVER_RULE_PKG:
-            return std::string(problemDict[RULE_PKG]);
+            return std::string(TM_(problemDict.at(RULE_PKG), 1));
         case SOLVER_RULE_BEST:
             if (source > 0)
-                return std::string(problemDict[RULE_BEST_1]) + pool_solvid2str(pool, source);
-            return std::string(problemDict[RULE_BEST_2]);
+                return std::string(TM_(problemDict.at(RULE_BEST_1), 1)) + solvid2str(pool, source);
+            return std::string(TM_(problemDict.at(RULE_BEST_2), 1));
         case SOLVER_RULE_PKG_NOT_INSTALLABLE:
             ss = pool->solvables + source;
             if (pool_disabled_solvable(pool, ss)) {
                 if (modularExclude && modularExclude->has(source)) {
-                    return tfm::format(problemDict[RULE_PKG_NOT_INSTALLABLE_1], pool_solvid2str(
-                        pool, source));
+                    return tfm::format(TM_(problemDict.at(RULE_PKG_NOT_INSTALLABLE_1), 1),
+                                       solvid2str(pool, source).c_str(), pool_id2solvable(pool, source)->repo->name);
                 } else {
-                    return tfm::format(problemDict[RULE_PKG_NOT_INSTALLABLE_4], pool_solvid2str(
-                        pool, source));
+                    return tfm::format(TM_(problemDict.at(RULE_PKG_NOT_INSTALLABLE_4), 1),
+                                       solvid2str(pool, source).c_str(), pool_id2solvable(pool, source)->repo->name);
                 }
             }
             if (ss->arch && ss->arch != ARCH_SRC && ss->arch != ARCH_NOSRC &&
                 pool->id2arch && (ss->arch > pool->lastarch || !pool->id2arch[ss->arch]))
-                return tfm::format(problemDict[RULE_PKG_NOT_INSTALLABLE_2], pool_solvid2str(pool, source));
-            return tfm::format(problemDict[RULE_PKG_NOT_INSTALLABLE_3], pool_solvid2str(pool, source));
+                return tfm::format(TM_(problemDict.at(RULE_PKG_NOT_INSTALLABLE_2), 1), solvid2str(pool, source).c_str(),
+                                   pool_id2solvable(pool, source)->repo->name);
+            return tfm::format(TM_(problemDict.at(RULE_PKG_NOT_INSTALLABLE_3), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name);
         case SOLVER_RULE_PKG_NOTHING_PROVIDES_DEP:
-            return tfm::format(problemDict[RULE_PKG_NOTHING_PROVIDES_DEP], pool_dep2str(pool, dep),
-                               pool_solvid2str(pool, source));
+            return tfm::format(TM_(problemDict.at(RULE_PKG_NOTHING_PROVIDES_DEP), 1), pool_dep2str(pool, dep),
+                               solvid2str(pool, source).c_str(), pool_id2solvable(pool, source)->repo->name);
         case SOLVER_RULE_PKG_SAME_NAME:
-            return tfm::format(problemDict[RULE_PKG_SAME_NAME], pool_solvid2str(pool, source),
-                               pool_solvid2str(pool, target));
+            return tfm::format(TM_(problemDict.at(RULE_PKG_SAME_NAME), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name, solvid2str(pool, target).c_str(),
+                               pool_id2solvable(pool, target)->repo->name);
         case SOLVER_RULE_PKG_CONFLICTS:
-            return tfm::format(problemDict[RULE_PKG_CONFLICTS], pool_solvid2str(pool, source),
-                               pool_dep2str(pool, dep), pool_solvid2str(pool, target));
+            return tfm::format(TM_(problemDict.at(RULE_PKG_CONFLICTS), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name, pool_dep2str(pool, dep),
+                               solvid2str(pool, target).c_str(), pool_id2solvable(pool, target)->repo->name);
         case SOLVER_RULE_PKG_OBSOLETES:
-            return tfm::format(problemDict[RULE_PKG_OBSOLETES], pool_solvid2str(pool, source),
-                               pool_dep2str(pool, dep), pool_solvid2str(pool, target));
+            return tfm::format(TM_(problemDict.at(RULE_PKG_OBSOLETES), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name, pool_dep2str(pool, dep),
+                               solvid2str(pool, target).c_str(), pool_id2solvable(pool, target)->repo->name);
         case SOLVER_RULE_PKG_INSTALLED_OBSOLETES:
-            return tfm::format(problemDict[RULE_PKG_INSTALLED_OBSOLETES],
-                               pool_solvid2str(pool, source), pool_dep2str(pool, dep),
-                               pool_solvid2str(pool, target));
+            return tfm::format(TM_(problemDict.at(RULE_PKG_INSTALLED_OBSOLETES), 1),
+                               solvid2str(pool, source).c_str(), pool_dep2str(pool, dep),
+                               solvid2str(pool, target).c_str(), pool_id2solvable(pool, target)->repo->name);
         case SOLVER_RULE_PKG_IMPLICIT_OBSOLETES:
-            return tfm::format(problemDict[RULE_PKG_IMPLICIT_OBSOLETES],
-                               pool_solvid2str(pool, source), pool_dep2str(pool, dep),
-                               pool_solvid2str(pool, target));
+            return tfm::format(TM_(problemDict.at(RULE_PKG_IMPLICIT_OBSOLETES), 1),
+                               solvid2str(pool, source).c_str(), pool_dep2str(pool, dep),
+                               pool_id2solvable(pool, source)->repo->name, solvid2str(pool, target).c_str(),
+                               pool_id2solvable(pool, target)->repo->name);
         case SOLVER_RULE_PKG_REQUIRES:
-            return tfm::format(problemDict[RULE_PKG_REQUIRES], pool_solvid2str(pool, source),
-                               pool_dep2str(pool, dep));
+            return tfm::format(TM_(problemDict.at(RULE_PKG_REQUIRES), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name, pool_dep2str(pool, dep));
         case SOLVER_RULE_PKG_SELF_CONFLICT:
-            return tfm::format(problemDict[RULE_PKG_SELF_CONFLICT], pool_solvid2str(pool, source),
-                               pool_dep2str(pool, dep));
+            return tfm::format(TM_(problemDict.at(RULE_PKG_SELF_CONFLICT), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name, pool_dep2str(pool, dep));
         case SOLVER_RULE_YUMOBS:
-            return tfm::format(problemDict[RULE_YUMOBS], pool_solvid2str(pool, source),
-                               pool_solvid2str(pool, target), pool_dep2str(pool, dep));
+            return tfm::format(TM_(problemDict.at(RULE_YUMOBS), 1), solvid2str(pool, source).c_str(),
+                               pool_id2solvable(pool, source)->repo->name, solvid2str(pool, target).c_str(),
+                               pool_id2solvable(pool, target)->repo->name, pool_dep2str(pool, dep));
         default:
             return solver_problemruleinfo2str(solv, type, source, target, dep);
     }
@@ -497,11 +531,11 @@ static bool
 */
 can_depend_on(Pool *pool, Solvable *sa, Id b)
 {
-    IdQueue requires;
+    IdQueue dep_requires;
 
-    solvable_lookup_idarray(sa, SOLVABLE_REQUIRES, requires.getQueue());
-    for (int i = 0; i < requires.size(); ++i) {
-        Id req_dep = requires[i];
+    solvable_lookup_idarray(sa, SOLVABLE_REQUIRES, dep_requires.getQueue());
+    for (int i = 0; i < dep_requires.size(); ++i) {
+        Id req_dep = dep_requires[i];
         Id p, pp;
 
         FOR_PROVIDES(p, pp, req_dep)
@@ -609,10 +643,16 @@ erase_flags2libsolv(int flags)
     return ret;
 }
 
+static bool
+NameSolvableComparator(const Solvable * first, const Solvable * second)
+{
+    return first->name < second->name;
+}
+
 Goal::Goal(const Goal & goal_src) : pImpl(new Impl(*goal_src.pImpl)) {}
 
 Goal::Impl::Impl(const Goal::Impl & goal_src)
-: sack(goal_src.sack)
+: sack(goal_src.sack), exclude_from_weak(goal_src.exclude_from_weak)
 {
     queue_init_clone(&staging, const_cast<Queue *>(&goal_src.staging));
 
@@ -626,7 +666,7 @@ Goal::Impl::Impl(const Goal::Impl & goal_src)
 }
 
 Goal::Impl::Impl(DnfSack *sack)
-: sack(sack)
+: sack(sack), exclude_from_weak(sack)
 {
     queue_init(&staging);
 }
@@ -770,6 +810,108 @@ Goal::favor(DnfPackage *pkg)
     queue_push2(&pImpl->staging, SOLVER_SOLVABLE|SOLVER_FAVOR, dnf_package_get_id(pkg));
 }
 
+void
+Goal::add_exclude_from_weak(const DnfPackageSet & pset)
+{
+    pImpl->exclude_from_weak += pset;
+}
+
+void
+Goal::add_exclude_from_weak(DnfPackage *pkg)
+{
+    // ensure that the map has a corrent size before set to prevent memory corruption
+    map_grow(pImpl->exclude_from_weak.getMap(), dnf_sack_get_pool(pImpl->sack)->nsolvables);
+    pImpl->exclude_from_weak.set(pkg);
+}
+
+void
+Goal::reset_exclude_from_weak()
+{
+    pImpl->exclude_from_weak.clear();
+}
+
+void
+Goal::exclude_from_weak_autodetect()
+{
+    Query installed_query(pImpl->sack, Query::ExcludeFlags::IGNORE_EXCLUDES);
+    installed_query.installed();
+    if (installed_query.empty()) {
+        return;
+    }
+    Query base_query(pImpl->sack);
+    base_query.apply();
+    auto * installed_pset = installed_query.getResultPset();
+    Id installed_id = -1;
+
+    std::vector<const char *> installed_names;
+    installed_names.reserve(installed_pset->size() + 1);
+
+    // Iterate over installed packages to detect unmet weak deps
+    while ((installed_id = installed_pset->next(installed_id)) != -1) {
+        g_autoptr(DnfPackage) pkg = dnf_package_new(pImpl->sack, installed_id);
+        installed_names.push_back(dnf_package_get_name(pkg));
+        std::unique_ptr<libdnf::DependencyContainer> recommends(dnf_package_get_recommends(pkg));
+        for (int i = 0; i < recommends->count(); ++i) {
+            std::unique_ptr<libdnf::Dependency> dep(recommends->getPtr(i));
+            const char * dep_string = dep->toString();
+            if (dep_string[0] == '(') {
+                continue;
+            }
+            Query query(base_query);
+            const char * version = dep->getVersion();
+            //  There can be installed provider in different version or upgraded packed can recommend a different version
+            //  Ignore version and search only by reldep name
+            if (version && strlen(version) > 0) {
+                query.addFilter(HY_PKG_PROVIDES, HY_EQ, dep->getName());
+            } else {
+                query.addFilter(HY_PKG_PROVIDES, dep.get());
+            }
+            // No providers of recommend => continue
+            if (query.empty()) {
+                continue;
+            }
+            Query test_installed(query);
+            test_installed.installed();
+            // when there is not installed any provider of recommend, exclude it
+            if (test_installed.empty()) {
+                add_exclude_from_weak(*query.getResultPset());
+            }
+        }
+    }
+
+    // Investigate supplements of only available packages with a different name to installed packages
+    installed_names.push_back(nullptr);
+    base_query.addFilter(HY_PKG_NAME, HY_NEQ, installed_names.data());
+    auto * available_pset = base_query.getResultPset();
+    *available_pset -= *installed_pset;
+    Id available_id = -1;
+    while ((available_id = available_pset->next(available_id)) != -1) {
+        g_autoptr(DnfPackage) pkg = dnf_package_new(pImpl->sack, available_id);
+        std::unique_ptr<libdnf::DependencyContainer> supplements(dnf_package_get_supplements(pkg));
+        if (supplements->count() == 0) {
+            continue;
+        }
+        libdnf::DependencyContainer supplements_without_rich(getSack());
+        for (int i = 0; i < supplements->count(); ++i) {
+            std::unique_ptr<libdnf::Dependency> dep(supplements->getPtr(i));
+            const char * dep_string = dep->toString();
+            if (dep_string[0] == '(') {
+                continue;
+            }
+            supplements_without_rich.add(dep.get());
+        }
+        if (supplements_without_rich.count() == 0) {
+            continue;
+        }
+        Query query(installed_query);
+        query.addFilter(HY_PKG_PROVIDES, &supplements_without_rich);
+        // When supplemented package already installed, exclude_from_weak available package
+        if (!query.empty()) {
+            add_exclude_from_weak(pkg);
+        }
+    }
+}
+
 void
 Goal::disfavor(DnfPackage *pkg)
 {
@@ -973,12 +1115,6 @@ Goal::describeProblemRules(unsigned i, bool pkgs)
     IdQueue rq;
     // this libsolv interface indexes from 1 (we do from 0), so:
     solver_findallproblemrules(solv, i+1, pq.getQueue());
-    std::map<int, const char *> problemDict;
-    if (pkgs) {
-        problemDict = PKG_PROBLEMS_DICT;
-    } else {
-        problemDict = MODULE_PROBLEMS_DICT;
-    }
     std::unique_ptr<libdnf::PackageSet> modularExcludes(dnf_sack_get_module_excludes(pImpl->sack));
     for (j = 0; j < pq.size(); j++) {
         rid = pq[j];
@@ -989,7 +1125,7 @@ Goal::describeProblemRules(unsigned i, bool pkgs)
                 target = rq[ir + 2];
                 dep = rq[ir + 3];
                 auto problem_str = libdnf_problemruleinfo2str(modularExcludes.get(), solv, type,
-                                                              source, target, dep, problemDict);
+                                                              source, target, dep, pkgs);
                 unique = true;
                 for (auto & item: output) {
                     if (problem_str == item) {
@@ -1006,6 +1142,49 @@ Goal::describeProblemRules(unsigned i, bool pkgs)
     return output;
 }
 
+bool
+Goal::isBrokenFileDependencyPresent()
+{
+    return pImpl->isBrokenFileDependencyPresent();
+}
+
+bool
+Goal::Impl::isBrokenFileDependencyPresent()
+{
+    for (int i = 0; i < countProblems(); i++) {
+        if (isBrokenFileDependencyPresent(i)) {
+            return true;
+        }
+    }
+    return false;
+}
+
+bool
+Goal::Impl::isBrokenFileDependencyPresent(unsigned i)
+{
+    if (i >= solver_problem_count(solv)) {
+        return false;
+    }
+
+    SolverRuleinfo type;
+    Id source, target, dep;
+    IdQueue pq;
+    Pool * const pool = solv->pool;
+
+    // this libsolv interface indexes from 1 (we do from 0), so:
+    solver_findallproblemrules(solv, i+1, pq.getQueue());
+    for (int j = 0; j < pq.size(); j++) {
+        type = solver_ruleinfo(solv, pq[j], &source, &target, &dep);
+        if (type == SOLVER_RULE_PKG_NOTHING_PROVIDES_DEP) {
+            auto dependency = std::string(pool_dep2str(pool, dep));
+            if (dependency.at(0) == '/') {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
 /**
  * Write all the solving decisions to the hawkey logfile.
  */
@@ -1244,6 +1423,12 @@ Goal::Impl::constructJob(DnfGoalActions flags)
             elements[i] |= SOLVER_FORCEBEST;
     }
 
+    // Add weak excludes to the job
+    Id id = -1;
+    while ((id = exclude_from_weak.next(id)) != -1) {
+        job->pushBack(SOLVER_SOLVABLE|SOLVER_EXCLUDEFROMWEAK, id);
+    }
+
     /* turn off implicit obsoletes for installonly packages */
     for (int i = 0; i < (int) dnf_sack_get_installonly(sack)->count; i++)
         job->pushBack(SOLVER_MULTIVERSION|SOLVER_SOLVABLE_PROVIDES,
@@ -1300,10 +1485,24 @@ Goal::Impl::limitInstallonlyPackages(Solver *solv, Queue *job)
     for (int i = 0; i < onlies->count; ++i) {
         Id p, pp;
         IdQueue q, installing;
+        std::vector<Solvable *> available_unused_providers;
 
+        // Add all providers of installonly provides that are marked for install
+        // to `q` IdQueue those that are not marked for install and are not already
+        // installed are added to available_unused_providers.
         FOR_PKG_PROVIDES(p, pp, onlies->elements[i])
-            if (solver_get_decisionlevel(solv, p) > 0)
+            // According to libsolv-bindings the decision level is positive for installs
+            // and negative for conflicts (conflicts with another package or dependency
+            // conflicts = dependencies cannot be met).
+            if (solver_get_decisionlevel(solv, p) > 0) {
                 q.pushBack(p);
+            } else {
+                Solvable *s = pool_id2solvable(pool, p);
+                if (s->repo != pool->installed) {
+                    available_unused_providers.push_back(s);
+                }
+            }
+
         if (q.size() <= (int) dnf_sack_get_installonly_limit(sack)) {
             continue;
         }
@@ -1321,6 +1520,7 @@ Goal::Impl::limitInstallonlyPackages(Solver *solv, Queue *job)
 
         struct InstallonliesSortCallback s_cb = {pool, dnf_sack_running_kernel(sack)};
         solv_sort(q.data(), q.size(), sizeof(q[0]), sort_packages, &s_cb);
+        std::sort(available_unused_providers.begin(), available_unused_providers.end(), NameSolvableComparator);
         IdQueue same_names;
         while (q.size() > 0) {
             same_name_subqueue(pool, q.getQueue(), same_names.getQueue());
@@ -1330,8 +1530,18 @@ Goal::Impl::limitInstallonlyPackages(Solver *solv, Queue *job)
             for (int j = 0; j < same_names.size(); ++j) {
                 Id id  = same_names[j];
                 Id action = SOLVER_ERASE;
-                if (j < (int) dnf_sack_get_installonly_limit(sack))
+                if (j < (int) dnf_sack_get_installonly_limit(sack)) {
                     action = SOLVER_INSTALL;
+                } else  {
+                    // We want to avoid reinstalling packages marked for ERASE, therefore
+                    // if some unused provider is also available we need to mark it ERASE as well.
+                    Solvable *s = pool_id2solvable(pool, id);
+                    auto low = std::lower_bound(available_unused_providers.begin(), available_unused_providers.end(), s, NameSolvableComparator);
+                    while (low != available_unused_providers.end() && (*low)->name == s->name) {
+                        queue_push2(job, SOLVER_ERASE | SOLVER_SOLVABLE, pool_solvable2id(pool, *low));
+                        ++low;
+                    }
+                }
                 queue_push2(job, action | SOLVER_SOLVABLE, id);
             }
         }
@@ -1495,12 +1705,24 @@ Goal::Impl::protectedInRemovals()
     if ((!protectedPkgs || !protectedPkgs->size()) && !protect_running_kernel)
         return false;
     auto pkgRemoveList = listResults(SOLVER_TRANSACTION_ERASE, 0);
+    Id protected_kernel = protectedRunningKernel();
     auto pkgObsoleteList = listResults(SOLVER_TRANSACTION_OBSOLETED, 0);
-    map_or(pkgRemoveList.getMap(), pkgObsoleteList.getMap());
+    // Special case: consider the obsoletion of the running kernel as a
+    // removal. Obsoletion of other protected packages should be allowed.
+    for (size_t obsolete_index = 0; obsolete_index < pkgObsoleteList.size(); obsolete_index += 1) {
+        if (protected_kernel == pkgObsoleteList[obsolete_index]) {
+            pkgRemoveList.set(protected_kernel);
+        }
+    }
 
+    // We want to allow obsoletion of protected packages, so we do not consider
+    // obsoletes here, only removes. Previously, obsoletion of protected
+    // packages was disallowed, but there needed to be some mechanism for
+    // obsoleting/swapping a protected package, such as to obsolete `dnf` in
+    // favor of `dnf5`. Obsoleting a package is much harder to do accidentally
+    // than removing it.
     removalOfProtected.reset(new PackageSet(pkgRemoveList));
     Id id = -1;
-    Id protected_kernel = protectedRunningKernel();
     while(true) {
         id = removalOfProtected->next(id);
         if (id == -1)
