@@ -189,6 +189,13 @@ NameArchSolvableComparator(const Solvable * first, const Solvable * second)
     return first->arch < second->arch;
 }
 
+static bool
+NameSolvableComparator(const Solvable * first, const Solvable * second)
+{
+    return first->name < second->name;
+}
+
+
 static bool
 NamePrioritySolvableKey(const Solvable * first, const Solvable * second)
 {
@@ -1662,7 +1669,7 @@ Query::Impl::filterObsoletes(const Filter & f, Map *m)
         Solvable *s = pool_id2solvable(pool, id);
         if (!s->repo)
             continue;
-        for (Id *r_id = s->repo->idarraydata + s->obsoletes; *r_id; ++r_id) {
+        for (Id *r_id = s->repo->idarraydata + s->dep_obsoletes; *r_id; ++r_id) {
             Id r, rr;
 
             FOR_PROVIDES(r, rr, *r_id) {
@@ -1684,7 +1691,7 @@ Query::Impl::obsoletesByPriority(Pool * pool, Solvable * candidate, Map * m, con
 {
     if (!candidate->repo)
         return;
-    for (Id *r_id = candidate->repo->idarraydata + candidate->obsoletes; *r_id; ++r_id) {
+    for (Id *r_id = candidate->repo->idarraydata + candidate->dep_obsoletes; *r_id; ++r_id) {
         Id r, rr;
         FOR_PROVIDES(r, rr, *r_id) {
             if (!MAPTST(target, r))
@@ -1878,11 +1885,14 @@ Query::Impl::filterAdvisory(const Filter & f, Map *m, int keyname)
         std::vector<Solvable *> installed_solvables;
 
         if (cmp_type & HY_UPGRADE) {
-            // When doing HY_UPGRADE consider only candidate pkgs that have matching Name and Arch with:
-            // * some already installed pkg (in other words: some other version of the pkg is already installed)
-            // or
-            // * with pkg that obsoletes some already installed (or to be installed in this transaction) pkg
-            // Otherwise a pkg with different Arch than installed can end up in upgrade set which is wrong.
+            // When doing HY_UPGRADE consider only candidate pkgs that:
+            // * have matching Name and Arch with some already installed pkg
+            //   (in other words: some other version of the pkg is already installed)
+            // * have matching Name with some already installed pkg and either the candidate or the installed pkg is noarch.
+            //   This matches upgrade behavior where we allow architecture change only when noarch is involved.
+            //   Details: RhBug:2124483, RhBug:2101398 and RhBug:1171543
+            // * obsoletes some already installed (or to be installed in this transaction) pkg
+            // Otherwise a pkg with different Arch than installed (and than noarch) can end up in upgrade set which is wrong.
             // It can result in dependency issues, reported as: RhBug:2088149.
 
             Query installed(sack, ExcludeFlags::IGNORE_EXCLUDES);
@@ -1915,12 +1925,16 @@ Query::Impl::filterAdvisory(const Filter & f, Map *m, int keyname)
             }
 
             Id id = -1;
-            // Add to candidates resultPset pkgs that match name and arch with some already installed pkg
+            // Add to candidates resultPset pkgs that match name and arch with some already installed pkg or match name and either the installed or candidate are NOARCH
             while ((id = resultPset->next(id)) != -1) {
                 Solvable * s = pool_id2solvable(pool, id);
-                auto low = std::lower_bound(installed_solvables.begin(), installed_solvables.end(), s, NameArchSolvableComparator);
-                if (low != installed_solvables.end() && s->name == (*low)->name && s->arch == (*low)->arch) {
-                    candidates.push_back(s);
+                auto low = std::lower_bound(installed_solvables.begin(), installed_solvables.end(), s, NameSolvableComparator);
+                while (low != installed_solvables.end() && (*low)->name == s->name) {
+                    if (s->arch == (*low)->arch || s->arch == ARCH_NOARCH || (*low)->arch == ARCH_NOARCH) {
+                        candidates.push_back(s);
+                        break;
+                    }
+                    ++low;
                 }
             }
 
@@ -2299,7 +2313,7 @@ Query::Impl::apply()
     if (!result)
         initResult();
     map_init(&m, pool->nsolvables);
-    assert(m.size == result->getMap()->size);
+    map_grow(result->getMap(), pool->nsolvables);
     for (auto f : filters) {
         map_empty(&m);
         switch (f.getKeyname()) {
